#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include <stdio.h>
#include "Config.h"
#include "stm32f107.h"
#include "ulcd_defs.h"

static volatile uint32_t __counter;
static volatile uint32_t led_counter;

static inline void delay_ms(uint32_t ms){
  __counter = 0;
  while(__counter < ms * 10);
}

static void init();
void memclr(void *buffer, size_t size);

#define ulcd_delay()
#define ulcd_waitcmd()        while(bit_is_set(ulcd_regread(ULCD_REG_STATUS), 0))

uint32_t ulcd_regread(uint8_t index);
void ulcd_regwrite(uint8_t index, uint32_t value);
void ulcd_regmov(uint8_t to, uint8_t from);
void ulcd_cmd(uint8_t index, uint8_t *param, uint32_t count);

int main(){
  init(); // Инициализация переферийных устройств STM32

  delay_ms(500); // Инициализация ULCD
  ulcd_cmd(ULCD_CMD_SOFTRESET, 0, 0); // Программный сброс
  delay_ms(500);

  uint8_t param[13];

  uint32_t counter = 9999;
  char counter_str[5];
  sprintf(counter_str, "%lu", counter);

  // Создание контейнера со строкой
  ulcd_cmd(ULCD_CMD_BUFFER_CREATE, 0, 0);
  ulcd_waitcmd();
  uint32_t counter_buf = ulcd_regread(ULCD_REG_CURRENT);
  // Установка размера контейнера
  ulcd_regwrite(ULCD_REG_OPERAND0, sizeof(counter_str));
  ulcd_cmd(ULCD_CMD_BUFFER_SETSIZE, 0, 0);
  ulcd_waitcmd();
  // Запись строки в контейнер
  ulcd_regwrite(ULCD_REG_POINTER, 0);
  for(uint32_t i = 0; i < sizeof(counter_str); i++){
    ulcd_regwrite(ULCD_REG_OPERAND0, counter_str[i]);
    ulcd_cmd(ULCD_CMD_BUFFER_PUTBYTE, 0, 0);
    ulcd_waitcmd();
  }

  // Создаем фрейм
  param[0] = ULCD_WIDGET_FRAME;
  ulcd_cmd(ULCD_CMD_WIDGET_CREATE, param, 1);
  ulcd_waitcmd();
  uint32_t frame = ulcd_regread(ULCD_REG_CURRENT);
  ulcd_regwrite(ULCD_REG_R2, frame);

  // Создаем текстовую метку
  param[0] = ULCD_WIDGET_TEXT;
  ulcd_cmd(ULCD_CMD_WIDGET_CREATE, param, 1);
  ulcd_waitcmd();
  uint32_t text = ulcd_regread(ULCD_REG_CURRENT);
  // Установка шрифта
  ulcd_regmov(ULCD_REG_OPERAND0, ULCD_REG_SYSFONT);
  // ulcd_regwrite(ULCD_REG_OPERAND0, font);
  ulcd_cmd(ULCD_CMD_WIDGET_SETFONT, 0, 0);
  ulcd_waitcmd();
  // Установка данных
  ulcd_regwrite(ULCD_REG_OPERAND0, counter_buf);
  ulcd_cmd(ULCD_CMD_WIDGET_SETDATA, 0, 0);
  ulcd_waitcmd();
  // Чтение размеров виджета
  ulcd_cmd(ULCD_CMD_WIDGET_GETSIZE, 0, 0);
  ulcd_waitcmd();
  uint32_t width = ulcd_regread(ULCD_REG_RESULT0);
  uint32_t height = ulcd_regread(ULCD_REG_RESULT1);
  uint32_t ulcd_width = ulcd_regread(ULCD_REG_LCDWIDTH);
  uint32_t ulcd_height = ulcd_regread(ULCD_REG_LCDHEIGHT);
  // Устанавливаем координату
  ulcd_regwrite(ULCD_REG_OPERAND0, (ulcd_width - width) / 2); // X
  ulcd_regwrite(ULCD_REG_OPERAND1, (ulcd_height - height) / 2); // Y
  ulcd_cmd(ULCD_CMD_WIDGET_SETCOORD, 0, 0);
  ulcd_waitcmd();
  // Устанавливаем стиль
  ulcd_regwrite(ULCD_REG_OPERAND0, _BV(19)); // TEXT_NOAUTORESIZE
  ulcd_cmd(ULCD_CMD_WIDGET_SETSTYLE, 0, 0);
  ulcd_waitcmd();

  // Создаем Slider
  param[0] = ULCD_WIDGET_SLIDER;
  ulcd_cmd(ULCD_CMD_WIDGET_CREATE, param, 1);
  ulcd_waitcmd();
  uint32_t slider = ulcd_regread(ULCD_REG_CURRENT);
  // Устанавливаем координату
  ulcd_regwrite(ULCD_REG_OPERAND0, 10); // X
  ulcd_regwrite(ULCD_REG_OPERAND1, 10); // Y
  ulcd_cmd(ULCD_CMD_WIDGET_SETCOORD, 0, 0);
  ulcd_waitcmd();
  // Устанавливаем размер
  ulcd_regwrite(ULCD_REG_OPERAND0, ulcd_width - 20); // WIDTH
  ulcd_regwrite(ULCD_REG_OPERAND1, 30); // HEIGHT
  ulcd_cmd(ULCD_CMD_WIDGET_SETSIZE, 0, 0);
  ulcd_waitcmd();
  // Устанавливаем минимальную позицию
  ulcd_regwrite(ULCD_REG_OPERAND0, 10);
  ulcd_cmd(ULCD_CMD_WIDGET_SETMINPOSITION, 0, 0);
  ulcd_waitcmd();
  // Устанавливаем максимальную позицию
  ulcd_regwrite(ULCD_REG_OPERAND0, 1000);
  ulcd_cmd(ULCD_CMD_WIDGET_SETMAXPOSITION, 0, 0);
  ulcd_waitcmd();
  // Устанавливаем позицию
  ulcd_regwrite(ULCD_REG_OPERAND0, 1000);
  ulcd_cmd(ULCD_CMD_WIDGET_SETPOSITION, 0, 0);
  ulcd_waitcmd();
  // Устанавливаем шаг
  ulcd_regwrite(ULCD_REG_OPERAND0, 1);
  ulcd_cmd(ULCD_CMD_WIDGET_SETSTEP, 0, 0);
  ulcd_waitcmd();

  // Вывод на экран фрейма
  ulcd_regwrite(ULCD_REG_CURRENT, frame);
  ulcd_cmd(ULCD_CMD_SETFRAME, 0, 0);
  ulcd_waitcmd();

  bool ledison = false;
  led_counter = 0;

  while(true){
    // Моргание светодиодом
    if(led_counter > 500){
      if(ledison){
        GPIOC_BSRR = _BV(9);
      }
      else {
        GPIOC_BRR = _BV(9);
      }
      ledison = !ledison;
      led_counter = 0;
    }

    // Считываем позицию слайдера и записываем ее в метку.
    ulcd_regwrite(ULCD_REG_CURRENT, slider);
    ulcd_cmd(ULCD_CMD_WIDGET_GETPOSITION, 0, 0);
    ulcd_waitcmd();

    counter = ulcd_regread(ULCD_REG_RESULT0);
    sprintf(counter_str, "%lu", counter);

    // Запись строки в контейнер
    ulcd_regwrite(ULCD_REG_CURRENT, counter_buf);
    ulcd_regwrite(ULCD_REG_POINTER, 0);
    for(uint32_t i = 0; i < sizeof(counter_str); i++){
      ulcd_regwrite(ULCD_REG_OPERAND0, counter_str[i]);
      ulcd_cmd(ULCD_CMD_BUFFER_PUTBYTE, 0, 0);
      ulcd_waitcmd();
    }

    ulcd_regwrite(ULCD_REG_CURRENT, text);
    ulcd_cmd(ULCD_CMD_WIDGET_REFRESH, 0, 0);

    delay_ms(10);
  }
}

// Прерывание системного таймера (SysTick)
void __SysTick(){
  __counter++;
  led_counter++;
}

uint8_t spi_com(uint8_t data){
  GPIOD_BRR = _BV(7);
  while(bit_is_clear(SPI3_SR, SPI3_SR_TXE));
  SPI3_DR = data;
  while(bit_is_clear(SPI3_SR, SPI3_SR_RXNE));
  data = SPI3_DR;
  GPIOD_BSRR = _BV(7);
  return data;
}

uint32_t ulcd_regread(uint8_t index){
  spi_com(ULCD_CMD_REG_READ);
  ulcd_delay();
  spi_com(index);
  ulcd_delay();
  delay_ms(1);
  uint32_t result = 0;
  for(uint32_t i = 0; i < 4; i++){
    uint32_t data = (uint32_t)spi_com(0x00);
    result |= (data << (8 * i));
    ulcd_delay();
  }
  return result;
}

void ulcd_regwrite(uint8_t index, uint32_t value){
  spi_com(ULCD_CMD_REG_WRITE);
  ulcd_delay();
  spi_com(index);
  ulcd_delay();
  for(uint32_t i = 0; i < 4; i++){
    spi_com((value >> (8 * i)) & 0xFF);
    ulcd_delay();
  }
}

void ulcd_regmov(uint8_t to, uint8_t from){
  spi_com(ULCD_CMD_REG_MOVE);
  ulcd_delay();
  spi_com(to);
  ulcd_delay();
  spi_com(from);
  ulcd_delay();
}

void ulcd_cmd(uint8_t index, uint8_t *param, uint32_t count){
  spi_com(index);
  ulcd_delay();
  for(uint32_t i = 0; i < count; i++){
    spi_com(param[i]);
    ulcd_delay();
  }
}

// Инициализация переферийных устройств
static void init(){
  // Инициализация системного таймера
  SysTickReload = TIMRELOAD_CALC(100, 72000000); // 100 мкс ~ 10кГц
  SysTickCurrent = 0;
  SysTickControl = _BV(SysTickControl_ENABLE) | _BV(SysTickControl_TICKINT) |
  _BV(SysTickControl_CLKSOURCE); // Источник тактирования -- частота
  // процессора (AHB)

  RCC_APB2ENR |= _BV(RCC_APB2ENR_AFIOEN) | _BV(RCC_APB2ENR_IOPAEN) |
  _BV(RCC_APB2ENR_IOPBEN) | _BV(RCC_APB2ENR_IOPCEN) | _BV(RCC_APB2ENR_IOPDEN);
  
  // ULCD запрос на прерывание
  GPIOB_CRL = (GPIOB_CRL & ~((3 << GPIOB_CRL_CNF7) | (3 << GPIOB_CRL_MODE7))) |
  ((1 << GPIOB_CRL_CNF7) | (0 << GPIOB_CRL_MODE7)); // PB7 -- плавающий вход

  // Конфигурация выводов SPI и ULCD

  GPIOB_CRL = (GPIOB_CRL & ~((3 << GPIOB_CRL_CNF7) | (3 << GPIOB_CRL_MODE7))) |
  ((1 << GPIOB_CRL_CNF7) | (0 << GPIOB_CRL_MODE7)); // PB7, INT -- Плавающий
  // вход

  GPIOD_CRL = (GPIOD_CRL & ~((3 << GPIOD_CRL_CNF7) | (3 << GPIOD_CRL_MODE7))) |
  ((0 << GPIOD_CRL_CNF7) | (3 << GPIOD_CRL_MODE7)); // PD7, SSEL --
  // Двухтактный выход, 50МГц

  GPIOC_CRH = (GPIOC_CRH & ~((3 << GPIOC_CRH_CNF10) |
  (3 << GPIOC_CRH_MODE10))) | ((2 << GPIOC_CRH_CNF10) |
  (3 << GPIOC_CRH_MODE10)); // PC10, SCK -- Альтернативный двухтактный выход

  GPIOC_BSRR = _BV(11);
  GPIOC_CRH = (GPIOC_CRH & ~((3 << GPIOC_CRH_CNF11) |
  (3 << GPIOC_CRH_MODE11))) | ((2 << GPIOC_CRH_CNF11) |
  (0 << GPIOC_CRH_MODE11)); // PC11, MISO -- Подтянутый к питанию вход

  GPIOC_CRH = (GPIOC_CRH & ~((3 << GPIOC_CRH_CNF12) |
  (3 << GPIOC_CRH_MODE12))) | ((2 << GPIOC_CRH_CNF12) |
  (3 << GPIOC_CRH_MODE12)); // PC12, MOSI -- Альтернативный двухтактный выход

  AFIO_MAPR |= (1 << AFIO_MAPR_SPI3_REMAP);

  // Инициализация SPI
  RCC_APB1ENR |= _BV(RCC_APB1ENR_SPI3EN);
  SPI3_CR1 = (1 << SPI3_CR1_CPHA) | (1 << SPI3_CR1_CPOL) |
  (1 << SPI3_CR1_MSTR) | SPI3_CR1_BR_PCLK256_gc | (1 << SPI3_CR1_SPE) |
  (0 << SPI3_CR1_LSBFIRST) | (0 << SPI3_CR1_DFF) | (1 << SPI3_CR1_SSM) |
  (1 << SPI3_CR1_SSI);

  GPIOD_BSRR = _BV(7); // SSEL = 1

  // Инициализация светодиода (PC9)
  GPIOC_CRH = (GPIOC_CRH & ~(0xF << 4)) | (0x1 << 4); // Двухтактный
  // выход, 10МГц
  GPIOC_BRR = _BV(9); // PC9 = 0
}

void memclr(void *buffer, size_t size){
  for(uint32_t i = 0; i < size; i++){
    *((uint8_t *)buffer++) = 0;
  }
}

