#ifndef ULCD_DEFS_H
#define ULCD_DEFS_H

#define CMD_NOP                         00

// Операции с регистрами
#define CMD_REG_READ                    01
#define CMD_REG_WRITE                   02
#define CMD_REG_MOVE                    03
#define CMD_REG_INC                     04
#define CMD_REG_DEC                     05
#define CMD_REG_OR                      06
#define CMD_REG_AND                     07

// Операции с контейнерами
#define CMD_BUFFER_CREATE               08
#define CMD_BUFFER_DELETE               09
#define CMD_BUFFER_SETSIZE              0A
#define CMD_BUFFER_GETSIZE              0B
#define CMD_BUFFER_WRITE                0C
#define CMD_BUFFER_READ                 0D
#define CMD_BUFFER_LOAD                 0E
#define CMD_BUFFER_RUN                  0F
#define CMD_BUFFER_GETBYTE              10
#define CMD_BUFFER_PUTBYTE              11
#define CMD_BUFFER_GETSHORT             12
#define CMD_BUFFER_PUTSHORT             13
#define CMD_BUFFER_GETLONG              14
#define CMD_BUFFER_PUTLONG              15

// Операции со шрифтами
#define CMD_FONT_CREATE                 18
#define CMD_FONT_DELETE                 19

// Операции с виджетами
#define CMD_WIDGET_CREATE               20
#define CMD_WIDGET_DELETE               21
#define CMD_SETFRAME                    22
#define CMD_WIDGET_REFRESH              23
#define CMD_WIDGET_SETHANDLER           24
#define CMD_WIDGET_GETHANDLER           25
#define CMD_WIDGET_SETCOORD             26
#define CMD_WIDGET_GETCOORD             27
#define CMD_WIDGET_SETSIZE              28
#define CMD_WIDGET_GETSIZE              29
#define CMD_WIDGET_SETSTYLE             2A
#define CMD_WIDGET_GETSTYLE             2B
#define CMD_WIDGET_SETBGCOLOUR          2C
#define CMD_WIDGET_GETBGCOLOUR          2D
#define CMD_WIDGET_RESETBGCOLOUR        2E
#define CMD_WIDGET_SETTEXTCOLOUR        2F
#define CMD_WIDGET_GETTEXTCOLOUR        30
#define CMD_WIDGET_RESETTEXTCOLOUR      31
#define CMD_WIDGET_SETFONT              32
#define CMD_WIDGET_GETFONT              33
#define CMD_WIDGET_SETDATA              34
#define CMD_WIDGET_GETDATA              35
#define CMD_WIDGET_SETPOSITION          36
#define CMD_WIDGET_GETPOSITION          37

// Команды рисования
#define CMD_DRAW_POINT                  80
#define CMD_DRAW_LINE                   81
#define CMD_DRAW_RECTANGLE              82
#define CMD_FILL                        8C
#define CMD_DRAW_BITMAP                 8D
#define CMD_DRAW_TEXT                   8F
#define CMD_DRAW_ELLIPSE                92

// Прочие команды
#define CMD_SOFTRESET                   FE
#define CMD_TSCALIBRATION               FF

// Регистры общего назначения
#define REG_R0                          00
#define REG_R1                          01
#define REG_R2                          02
#define REG_R3                          03
#define REG_R4                          04
#define REG_R5                          05
#define REG_R6                          06
#define REG_R7                          07
#define REG_R8                          08
#define REG_R9                          09
#define REG_R10                         0A
#define REG_R11                         0B
#define REG_R12                         0C
#define REG_R13                         0D
#define REG_R14                         0E
#define REG_R15                         0F
#define REG_R16                         10
#define REG_R17                         11
#define REG_R18                         12
#define REG_R19                         13
#define REG_R20                         14
#define REG_R21                         15
#define REG_R22                         16
#define REG_R23                         17
#define REG_R24                         18
#define REG_R25                         19
#define REG_R26                         1A
#define REG_R27                         1B
#define REG_R28                         1C
#define REG_R29                         1D
#define REG_R30                         1E
#define REG_R31                         1F

// Общие регистры управления модулем
#define REG_STATUS                      21
#define REG_SYSFONT                     24
#define REG_LCDCONFIG                   28
#define REG_LCDWIDTH                    2A
#define REG_LCDHEIGHT                   2B
#define REG_GLOBAL_X                    2C
#define REG_GLOBAL_Y                    2D
#define REG_INTERRUPT                   30

// Регистры операций с виджетами
#define REG_PARENT                      40
#define REG_CURRENT                     41
#define REG_POINTER                     42
#define REG_OPERAND0                    48
#define REG_OPERAND1                    49
#define REG_OPERAND2                    4A
#define REG_OPERAND3                    4B
#define REG_OPERAND4                    4C
#define REG_OPERAND5                    4D
#define REG_OPERAND6                    4E
#define REG_OPERAND7                    4F
#define REG_RESULT0                     50
#define REG_RESULT1                     51
#define REG_ERROR                       58

// Регистры операций с виджетами из скриптов
#define REG_SCRPARENT                   80
#define REG_SCRCURRENT                  81
#define REG_SCRPOINTER                  82
#define REG_SCROPERAND0                 88
#define REG_SCROPERAND1                 89
#define REG_SCROPERAND2                 8A
#define REG_SCROPERAND3                 8B
#define REG_SCROPERAND4                 8C
#define REG_SCROPERAND5                 8D
#define REG_SCROPERAND6                 8E
#define REG_SCROPERAND7                 8F
#define REG_SCRRESULT0                  90
#define REG_SCRRESULT1                  91
#define REG_SCRERROR                    98

#define REG_SCR_ID                      A0
#define REG_SCR_X                       A1
#define REG_SCR_Y                       A2

// Регистры для рисования
#define REG_PEN                         C1
#define REG_BRUSH                       C3
#define REG_FONT                        C4
#define REG_ALPHA                       C5

// Операнды для загрузки данных
#define LOAD_FROM_FLASH                 00
#define LOAD_FROM_SD                    01

// Типы виджетов
#define WIDGET_WINDOW                   00
#define WIDGET_FRAME                    01
#define WIDGET_TIMER                    04
#define WIDGET_BITMAP                   05
#define WIDGET_PANEL                    06
#define WIDGET_TEXT                     07
#define WIDGET_BUTTON                   0A
#define WIDGET_ROTATECONTROL            0B
#define WIDGET_SLIDER                   0C
#define WIDGET_STATICLINE               10

// События
#define EVT_DOWN                        00
#define EVT_UP                          01
#define EVT_DCLICK                      02
#define EVT_HOLDDOWN                    03
#define EVT_MOTION                      04
#define EVT_ENTERWIN                    05
#define EVT_LEAVEWIN                    06
#define EVT_BUTTON                      07
#define EVT_DRAW                        08
#define EVT_ROTATE                      09
#define EVT_SLIDER                      0A

// Цвета
#define COLOUR_BLACK                    00000000
#define COLOUR_WHITE                    FFFFFF00
#define COLOUR_RED                      0000FF00
#define COLOUR_GREEN                    00FF0000
#define COLOUR_BLUE                     FF000000
#define COLOUR_YELLOW                   00FFFF00
#define COLOUR_CYAN                     FFFF0000
#define COLOUR_MAGENTA                  FF00FF00

#endif // ULCD_DEFS_H

