#ifndef STM32F107_H
#define STM32F107_H
#include <stdint.h>
#include <stdbool.h>
#include "SFR.h"

// Interrupt Control Type Register
#define IntCtrlType                     SFR_IO32(0xE000E004)
// SysTick
#define SysTickControl                  SFR_IO32(0xE000E010)
#define SysTickControl_ENABLE           (0)
#define SysTickControl_TICKINT          (1)
#define SysTickControl_CLKSOURCE        (2)
#define SysTickControl_COUNTFLAG        (16)
#define SysTickReload                   SFR_IO32(0xE000E014)
#define SysTickCurrent                  SFR_IO32(0xE000E018)
#define SysTickCalib                    SFR_IO32(0xE000E01C)
#define SysTickCalib_TENMS              (0)
#define SysTickCalib_SKEW               (30)
#define SysTickCalib_NOREF              (31)

// Vector Table Offset Register
#define VectorTableOffset               SFR_IO32(0xE000ED08)
#define VectorTableOffset_TBLOFF        (7)
#define VectorTableOffset_TBLBASE       (29)

// System Control Register
#define SystemControl                   SFR_IO32(0xE000ED10)

// SRAM
#define SRAM_BASE_ADDR                  (0x20000000)
#define SRAM_END_ADDR                   (0x2000FFFF)

//------------ Memory Map ------------------------------------------------------
// USB OTG FS
#define USB_OTG_FS_BASE_ADDR            (0x50000000)
// Ethernet
#define ETHERNET_BASE_ADDR              (0x40028000)
// CRC
#define CRC_BASE_ADDR                   (0x40023000)
// Flash memory interface
#define FLASH_IFACE_BASE_ADDR           (0x40022000)
#define FLASH_ACR                       SFR_IO32(FLASH_IFACE_BASE_ADDR + 0x000)
#define FLASH_ACR_LATENCY               (0)
#define FLASH_ACR_HLFCYA                (3)
#define FLASH_ACR_PRFTBE                (4)
#define FLASH_ACR_PRFTBS                (5)
#define FLASH_KEYR                      SFR_IO32(FLASH_IFACE_BASE_ADDR + 0x004)
#define FLASH_OPTKEYR                   SFR_IO32(FLASH_IFACE_BASE_ADDR + 0x008)
#define FLASH_SR                        SFR_IO32(FLASH_IFACE_BASE_ADDR + 0x00C)
#define FLASH_SR_BSY                    (0)
#define FLASH_SR_PGERR                  (2)
#define FLASH_SR_WRPRTERR               (4)
#define FLASH_SR_EOP                    (5)
#define FLASH_CR                        SFR_IO32(FLASH_IFACE_BASE_ADDR + 0x010)
#define FLASH_CR_PG                     (0)
#define FLASH_CR_PER                    (1)
#define FLASH_CR_MER                    (2)
#define FLASH_CR_OPTPG                  (4)
#define FLASH_CR_OPTER                  (5)
#define FLASH_CR_STRT                   (6)
#define FLASH_CR_LOCK                   (7)
#define FLASH_CR_OPTWRE                 (9)
#define FLASH_CR_ERRIE                  (10)
#define FLASH_CR_EOPIE                  (12)
#define FLASH_AR                        SFR_IO32(FLASH_IFACE_BASE_ADDR + 0x014)
#define FLASH_OBR                       SFR_IO32(FLASH_IFACE_BASE_ADDR + 0x01C)
#define FLASH_OBR_OPTERR                (0)
#define FLASH_OBR_RDPRT                 (1)
#define FLASH_OBR_WDG_SW                (2)
#define FLASH_OBR_nRST_STOP             (3)
#define FLASH_OBR_nRST_STDBY            (4)
#define FLASH_OBR_Data0                 (10)
#define FLASH_OBR_Data1                 (18)
#define FLASH_WRPR                      SFR_IO32(FLASH_IFACE_BASE_ADDR + 0x020)
// Reset and clock control RCC
#define RCC_BASE_ADDR                   (0x40021000)
#define RCC_CR                          SFR_IO32(RCC_BASE_ADDR + 0x000)
#define RCC_CR_HSION                    (0)
#define RCC_CR_HSIRDY                   (1)
#define RCC_CR_HSITRIM                  (3)
#define RCC_CR_HSICAL                   (8)
#define RCC_CR_HSEON                    (16)
#define RCC_CR_HSERDY                   (17)
#define RCC_CR_HSEBYP                   (18)
#define RCC_CR_CSSON                    (19)
#define RCC_CR_PLLON                    (24)
#define RCC_CR_PLLRDY                   (25)
#define RCC_CR_PLL2ON                   (26)
#define RCC_CR_PLL2RDY                  (27)
#define RCC_CR_PLL3ON                   (28)
#define RCC_CR_PLL3RDY                  (29)
#define RCC_CFGR                        SFR_IO32(RCC_BASE_ADDR + 0x004)
#define RCC_CFGR_SW                     (0)
#define RCC_CFGR_SWS                    (2)
#define RCC_CFGR_HPRE                   (4)
#define RCC_CFGR_PPRE1                  (8)
#define RCC_CFGR_PPRE2                  (11)
#define RCC_CFGR_ADCPRE                 (14)
#define RCC_CFGR_PLLSRC                 (16)
#define RCC_CFGR_PLLXTPRE               (17)
#define RCC_CFGR_PLLMUL                 (18)
#define RCC_CFGR_OTGFSPRE               (22)
#define RCC_CFGR_MCO                    (24)
#define RCC_CIR                         SFR_IO32(RCC_BASE_ADDR + 0x008)
#define RCC_CIR_LSIRDYF                 (0)
#define RCC_CIR_LSERDYF                 (1)
#define RCC_CIR_HSIRDYF                 (2)
#define RCC_CIR_HSERDYF                 (3)
#define RCC_CIR_PLLRDYF                 (4)
#define RCC_CIR_PLL2RDYF                (5)
#define RCC_CIR_PLL3RDYF                (6)
#define RCC_CIR_CSSF                    (7)
#define RCC_CIR_LSIRDYIE                (8)
#define RCC_CIR_LSERDYIE                (9)
#define RCC_CIR_HSIRDYIE                (10)
#define RCC_CIR_HSERDYIE                (11)
#define RCC_CIR_PLLRDYIE                (12)
#define RCC_CIR_PLL2RDYIE               (13)
#define RCC_CIR_PLL3RDYIE               (14)
#define RCC_CIR_LSIRDYC                 (16)
#define RCC_CIR_LSERDYC                 (17)
#define RCC_CIR_HSIRDYC                 (18)
#define RCC_CIR_HSERDYC                 (19)
#define RCC_CIR_PLLRDYC                 (20)
#define RCC_CIR_PLL2RDYC                (21)
#define RCC_CIR_PLL3RDYC                (22)
#define RCC_CIR_CSSC                    (23)
#define RCC_APB2RSTR                    SFR_IO32(RCC_BASE_ADDR + 0x00C)
#define RCC_APB2RSTR_AFIORST            (0)
#define RCC_APB2RSTR_IOPARST            (2)
#define RCC_APB2RSTR_IOPBRST            (3)
#define RCC_APB2RSTR_IOPCRST            (4)
#define RCC_APB2RSTR_IOPDRST            (5)
#define RCC_APB2RSTR_IOPERST            (6)
#define RCC_APB2RSTR_ADC1RST            (9)
#define RCC_APB2RSTR_ADC2RST            (10)
#define RCC_APB2RSTR_TIM1RST            (11)
#define RCC_APB2RSTR_SPI1RST            (12)
#define RCC_APB2RSTR_USART1RST          (14)
#define RCC_APB1RSTR                    SFR_IO32(RCC_BASE_ADDR + 0x010)
#define RCC_APB1RSTR_TIM2RST            (0)
#define RCC_APB1RSTR_TIM3RST            (1)
#define RCC_APB1RSTR_TIM4RST            (2)
#define RCC_APB1RSTR_TIM5RST            (3)
#define RCC_APB1RSTR_TIM6RST            (4)
#define RCC_APB1RSTR_TIM7RST            (5)
#define RCC_APB1RSTR_WWDGRST            (11)
#define RCC_APB1RSTR_SPI2RST            (14)
#define RCC_APB1RSTR_SPI3RST            (15)
#define RCC_APB1RSTR_USART2RST          (17)
#define RCC_APB1RSTR_USART3RST          (18)
#define RCC_APB1RSTR_UART4RST           (19)
#define RCC_APB1RSTR_UART5RST           (20)
#define RCC_APB1RSTR_I2C1RST            (21)
#define RCC_APB1RSTR_I2C2RST            (22)
#define RCC_APB1RSTR_CAN1RST            (25)
#define RCC_APB1RSTR_CAN2RST            (26)
#define RCC_APB1RSTR_BKPRST             (27)
#define RCC_APB1RSTR_PWRRST             (28)
#define RCC_APB1RSTR_DACRST             (29)
#define RCC_AHBENR                      SFR_IO32(RCC_BASE_ADDR + 0x014)
#define RCC_AHBENR_DMA1EN               (0)
#define RCC_AHBENR_DMA2EN               (1)
#define RCC_AHBENR_SRAMEN               (2)
#define RCC_AHBENR_FLITFEN              (4)
#define RCC_AHBENR_CRCEN                (6)
#define RCC_AHBENR_OTGFSEN              (12)
#define RCC_AHBENR_ETHMACEN             (14)
#define RCC_AHBENR_ETHMACTXEN           (15)
#define RCC_AHBENR_ETHMACRXEN           (16)
#define RCC_APB2ENR                     SFR_IO32(RCC_BASE_ADDR + 0x018)
#define RCC_APB2ENR_AFIOEN              (0)
#define RCC_APB2ENR_IOPAEN              (2)
#define RCC_APB2ENR_IOPBEN              (3)
#define RCC_APB2ENR_IOPCEN              (4)
#define RCC_APB2ENR_IOPDEN              (5)
#define RCC_APB2ENR_IOPEEN              (6)
#define RCC_APB2ENR_IOPFEN              (7)
#define RCC_APB2ENR_IOPGEN              (8)
#define RCC_APB2ENR_ADC1EN              (9)
#define RCC_APB2ENR_ADC2EN              (10)
#define RCC_APB2ENR_TIM1EN              (11)
#define RCC_APB2ENR_SPI1EN              (12)
#define RCC_APB2ENR_TIM8EN              (13)
#define RCC_APB2ENR_USART1EN            (14)
#define RCC_APB2ENR_ADC3EN              (15)
#define RCC_APB1ENR                     SFR_IO32(RCC_BASE_ADDR + 0x01C)
#define RCC_APB1ENR_TIM2EN              (0)
#define RCC_APB1ENR_TIM3EN              (1)
#define RCC_APB1ENR_TIM4EN              (2)
#define RCC_APB1ENR_TIM5EN              (3)
#define RCC_APB1ENR_TIM6EN              (4)
#define RCC_APB1ENR_TIM7EN              (5)
#define RCC_APB1ENR_WWDGEN              (11)
#define RCC_APB1ENR_SPI2EN              (14)
#define RCC_APB1ENR_SPI3EN              (15)
#define RCC_APB1ENR_USART2EN            (17)
#define RCC_APB1ENR_USART3EN            (18)
#define RCC_APB1ENR_UART4EN             (19)
#define RCC_APB1ENR_UART5EN             (20)
#define RCC_APB1ENR_I2C1EN              (21)
#define RCC_APB1ENR_I2C2EN              (22)
#define RCC_APB1ENR_USBEN               (23)
#define RCC_APB1ENR_CANEN               (25)
#define RCC_APB1ENR_BKPEN               (27)
#define RCC_APB1ENR_PWREN               (28)
#define RCC_APB1ENR_DACEN               (29)
#define RCC_BDCR                        SFR_IO32(RCC_BASE_ADDR + 0x020)
#define RCC_BDCR_LSEON                  (0)
#define RCC_BDCR_LSERDY                 (1)
#define RCC_BDCR_LSEBYP                 (2)
#define RCC_BDCR_RTCSEL                 (8)
#define RCC_BDCR_RTCEN                  (15)
#define RCC_BDCR_BDRST                  (16)
#define RCC_CSR                         SFR_IO32(RCC_BASE_ADDR + 0x024)
#define RCC_CSR_LSION                   (0)
#define RCC_CSR_LSIRDY                  (1)
#define RCC_CSR_RMVF                    (24)
#define RCC_CSR_PINRSTF                 (26)
#define RCC_CSR_PORRSTF                 (27)
#define RCC_CSR_SFTRSTF                 (28)
#define RCC_CSR_IWDGRSTF                (29)
#define RCC_CSR_WWDGRSTF                (30)
#define RCC_CSR_LPWRRSTF                (31)
#define RCC_AHBRSTR                     SFR_IO32(RCC_BASE_ADDR + 0x028)
#define RCC_AHBRSTR_OTGFSRST            (12)
#define RCC_AHBRSTR_ETHMACRST           (14)
#define RCC_CFGR2                       SFR_IO32(RCC_BASE_ADDR + 0x02C)
#define RCC_CFGR2_PREDIV1               (0)
#define RCC_CFGR2_PREDIV2               (4)
#define RCC_CFGR2_PLL2MUL               (8)
#define RCC_CFGR2_PLL3MUL               (12)
#define RCC_CFGR2_PREDIV1SRC            (16)
#define RCC_CFGR2_I2S2SRC               (17)
#define RCC_CFGR2_I2S3SRC               (18)
// DMA2
#define DMA2_BASE_ADDR                  (0x40020400)
// DMA1
#define DMA1_BASE_ADDR                  (0x40020000)
// SDIO
#define SDIO_BASE_ADDR                  (0x40018000)
// ADC3
#define ADC3_BASE_ADDR                  (0x40013C00)
// USART1
#define USART1_BASE_ADDR                (0x40013800)
// TIM8 timer
#define TIM8_BASE_ADDR                  (0x40013400)
// SPI1
#define SPI1_BASE_ADDR                  (0x40013000)
// TIM1 timer
#define TIM1_BASE_ADDR                  (0x40012C00)
#define TIM1_CR1                        SFR_IO32(TIM1_BASE_ADDR + 0x000)
#define TIM1_CR1_CEN                    (0)
#define TIM1_CR1_UDIS                   (1)
#define TIM1_CR1_URS                    (2)
#define TIM1_CR1_OPM                    (3)
#define TIM1_CR1_DIR                    (4)
#define TIM1_CR1_CMS                    (5)
#define TIM1_CR1_ARPE                   (7)
#define TIM1_CR1_CKD                    (8)
#define TIM1_CR2                        SFR_IO32(TIM1_BASE_ADDR + 0x004)
#define TIM1_SMCR                       SFR_IO32(TIM1_BASE_ADDR + 0x008)
#define TIM1_DIER                       SFR_IO32(TIM1_BASE_ADDR + 0x00C)
#define TIM1_SR                         SFR_IO32(TIM1_BASE_ADDR + 0x010)
#define TIM1_EGR                        SFR_IO32(TIM1_BASE_ADDR + 0x014)
#define TIM1_EGR_UG                     (0)
#define TIM1_EGR_CC1G                   (1)
#define TIM1_EGR_CC2G                   (2)
#define TIM1_EGR_CC3G                   (3)
#define TIM1_EGR_CC4G                   (4)
#define TIM1_EGR_TG                     (6)
#define TIM1_CCMR1                      SFR_IO32(TIM1_BASE_ADDR + 0x018)
#define TIM1_CCMR1_CC1S                 (0)
#define TIM1_CCMR1_OC1FE                (2)
#define TIM1_CCMR1_OC1PE                (3)
#define TIM1_CCMR1_OC1M                 (4)
#define TIM1_CCMR1_OC1CE                (7)
#define TIM1_CCMR1_IC1PSC               (2)
#define TIM1_CCMR1_IC1F                 (4)
#define TIM1_CCMR1_CC2S                 (8)
#define TIM1_CCMR1_OC2FE                (10)
#define TIM1_CCMR1_OC2PE                (11)
#define TIM1_CCMR1_OC2M                 (12)
#define TIM1_CCMR1_OC2CE                (15)
#define TIM1_CCMR1_IC2PSC               (10)
#define TIM1_CCMR1_IC2F                 (12)
#define TIM1_CCMR2                      SFR_IO32(TIM1_BASE_ADDR + 0x01C)
#define TIM1_CCMR2_CC3S                 (0)
#define TIM1_CCMR2_OC3FE                (2)
#define TIM1_CCMR2_OC3PE                (3)
#define TIM1_CCMR2_OC3M                 (4)
#define TIM1_CCMR2_OC3CE                (7)
#define TIM1_CCMR2_IC3PSC               (2)
#define TIM1_CCMR2_IC3F                 (4)
#define TIM1_CCMR2_CC4S                 (8)
#define TIM1_CCMR2_OC4FE                (10)
#define TIM1_CCMR2_OC4PE                (11)
#define TIM1_CCMR2_OC4M                 (12)
#define TIM1_CCMR2_OC4CE                (15)
#define TIM1_CCMR2_IC4PSC               (10)
#define TIM1_CCMR2_IC4F                 (12)
#define TIM1_CCER                       SFR_IO32(TIM1_BASE_ADDR + 0x020)
#define TIM1_CCER_CC1E                  (0)
#define TIM1_CCER_CC1P                  (1)
#define TIM1_CCER_CC1NE                 (2)
#define TIM1_CCER_CC1NP                 (3)
#define TIM1_CCER_CC2E                  (4)
#define TIM1_CCER_CC2P                  (5)
#define TIM1_CCER_CC2NE                 (6)
#define TIM1_CCER_CC2NP                 (7)
#define TIM1_CCER_CC3E                  (8)
#define TIM1_CCER_CC3P                  (9)
#define TIM1_CCER_CC3NE                 (10)
#define TIM1_CCER_CC3NP                 (11)
#define TIM1_CCER_CC4E                  (12)
#define TIM1_CCER_CC4P                  (13)
#define TIM1_CNT                        SFR_IO32(TIM1_BASE_ADDR + 0x024)
#define TIM1_PSC                        SFR_IO32(TIM1_BASE_ADDR + 0x028)
#define TIM1_ARR                        SFR_IO32(TIM1_BASE_ADDR + 0x02C)
#define TIM1_RCR                        SFR_IO32(TIM1_BASE_ADDR + 0x030)
#define TIM1_CCR1                       SFR_IO32(TIM1_BASE_ADDR + 0x034)
#define TIM1_CCR2                       SFR_IO32(TIM1_BASE_ADDR + 0x038)
#define TIM1_CCR3                       SFR_IO32(TIM1_BASE_ADDR + 0x03C)
#define TIM1_CCR4                       SFR_IO32(TIM1_BASE_ADDR + 0x040)
#define TIM1_BDTR                       SFR_IO32(TIM1_BASE_ADDR + 0x044)
#define TIM1_BDTR_DTG                   (0)
#define TIM1_BDTR_LOCK                  (8)
#define TIM1_BDTR_OSSI                  (10)
#define TIM1_BDTR_OSSR                  (11)
#define TIM1_BDTR_BKE                   (12)
#define TIM1_BDTR_BKP                   (13)
#define TIM1_BDTR_AOE                   (14)
#define TIM1_BDTR_MOE                   (15)
#define TIM1_DCR                        SFR_IO32(TIM1_BASE_ADDR + 0x048)
#define TIM1_DMAR                       SFR_IO32(TIM1_BASE_ADDR + 0x04C)
// ADC2
#define ADC2_BASE_ADDR                  (0x40012800)
// ADC1
#define ADC1_BASE_ADDR                  (0x40012400)
#define ADC1_SR                         SFR_IO32(ADC1_BASE_ADDR + 0x000)
#define ADC1_SR_AWD                     (0)
#define ADC1_SR_EOC                     (1)
#define ADC1_SR_JEOC                    (2)
#define ADC1_SR_JSTRT                   (3)
#define ADC1_SR_STRT                    (4)
#define ADC1_CR1                        SFR_IO32(ADC1_BASE_ADDR + 0x004)
#define ADC1_CR1_AWDCH                  (0)
#define ADC1_CR1_EOCIE                  (5)
#define ADC1_CR1_AWDIE                  (6)
#define ADC1_CR1_JEOCIE                 (7)
#define ADC1_CR1_SCAN                   (8)
#define ADC1_CR1_AWDSGL                 (9)
#define ADC1_CR1_JAUTO                  (10)
#define ADC1_CR1_DISCEN                 (11)
#define ADC1_CR1_JDISCEN                (12)
#define ADC1_CR1_DISCNUM                (13)
#define ADC1_CR1_DUALMOD                (16)
#define ADC1_CR1_JAWDEN                 (22)
#define ADC1_CR1_AWDEN                  (23)
#define ADC1_CR2                        SFR_IO32(ADC1_BASE_ADDR + 0x008)
#define ADC1_CR2_ADON                   (0)
#define ADC1_CR2_CONT                   (1)
#define ADC1_CR2_CAL                    (2)
#define ADC1_CR2_RSTCAL                 (3)
#define ADC1_CR2_DMA                    (8)
#define ADC1_CR2_ALIGN                  (11)
#define ADC1_CR2_JEXTSEL                (12)
#define ADC1_CR2_JEXTTRIG               (15)
#define ADC1_CR2_EXTSEL                 (17)
#define ADC1_CR2_EXTTRIG                (20)
#define ADC1_CR2_JSWSTART               (21)
#define ADC1_CR2_SWSTART                (22)
#define ADC1_CR2_TSVREFE                (23)
#define ADC1_SMPR1                      SFR_IO32(ADC1_BASE_ADDR + 0x00C)
#define ADC1_SMPR1_SMP10                (0)
#define ADC1_SMPR1_SMP11                (3)
#define ADC1_SMPR1_SMP12                (6)
#define ADC1_SMPR1_SMP13                (9)
#define ADC1_SMPR1_SMP14                (12)
#define ADC1_SMPR1_SMP15                (15)
#define ADC1_SMPR1_SMP16                (18)
#define ADC1_SMPR1_SMP17                (21)
#define ADC1_SMPR2                      SFR_IO32(ADC1_BASE_ADDR + 0x010)
#define ADC1_SMPR2_SMP0                 (0)
#define ADC1_SMPR2_SMP1                 (3)
#define ADC1_SMPR2_SMP2                 (6)
#define ADC1_SMPR2_SMP3                 (9)
#define ADC1_SMPR2_SMP4                 (12)
#define ADC1_SMPR2_SMP5                 (15)
#define ADC1_SMPR2_SMP6                 (18)
#define ADC1_SMPR2_SMP7                 (21)
#define ADC1_SMPR2_SMP8                 (24)
#define ADC1_SMPR2_SMP9                 (27)
#define ADC1_JOFR1                      SFR_IO32(ADC1_BASE_ADDR + 0x014)
#define ADC1_JOFR2                      SFR_IO32(ADC1_BASE_ADDR + 0x018)
#define ADC1_JOFR3                      SFR_IO32(ADC1_BASE_ADDR + 0x01C)
#define ADC1_JOFR4                      SFR_IO32(ADC1_BASE_ADDR + 0x020)
#define ADC1_HTR                        SFR_IO32(ADC1_BASE_ADDR + 0x024)
#define ADC1_LTR                        SFR_IO32(ADC1_BASE_ADDR + 0x028)
#define ADC1_SQR1                       SFR_IO32(ADC1_BASE_ADDR + 0x02C)
#define ADC1_SQR1_SQ13                  (0)
#define ADC1_SQR1_SQ14                  (5)
#define ADC1_SQR1_SQ15                  (10)
#define ADC1_SQR1_SQ16                  (15)
#define ADC1_SQR1_L                     (20)
#define ADC1_SQR2                       SFR_IO32(ADC1_BASE_ADDR + 0x030)
#define ADC1_SQR2_SQ7                   (0)
#define ADC1_SQR2_SQ8                   (5)
#define ADC1_SQR2_SQ9                   (10)
#define ADC1_SQR2_SQ10                  (15)
#define ADC1_SQR2_SQ11                  (20)
#define ADC1_SQR2_SQ12                  (25)
#define ADC1_SQR3                       SFR_IO32(ADC1_BASE_ADDR + 0x034)
#define ADC1_SQR3_SQ1                   (0)
#define ADC1_SQR3_SQ2                   (5)
#define ADC1_SQR3_SQ3                   (10)
#define ADC1_SQR3_SQ4                   (15)
#define ADC1_SQR3_SQ5                   (20)
#define ADC1_SQR3_SQ6                   (25)
#define ADC1_JSQR                       SFR_IO32(ADC1_BASE_ADDR + 0x038)
#define ADC1_JDR1                       SFR_IO32(ADC1_BASE_ADDR + 0x03C)
#define ADC1_JDR2                       SFR_IO32(ADC1_BASE_ADDR + 0x040)
#define ADC1_JDR3                       SFR_IO32(ADC1_BASE_ADDR + 0x044)
#define ADC1_JDR4                       SFR_IO32(ADC1_BASE_ADDR + 0x048)
#define ADC1_DR                         SFR_IO32(ADC1_BASE_ADDR + 0x04C)
// GPIO Port G
#define GPIOG_BASE_ADDR                 (0x40012000)
#define GPIOG_CRL                       SFR_IO32(GPIOG_BASE_ADDR + 0x000)
#define GPIOG_CRL_MODE0                 (0)
#define GPIOG_CRL_CNF0                  (2)
#define GPIOG_CRL_MODE1                 (4)
#define GPIOG_CRL_CNF1                  (6)
#define GPIOG_CRL_MODE2                 (8)
#define GPIOG_CRL_CNF2                  (10)
#define GPIOG_CRL_MODE3                 (12)
#define GPIOG_CRL_CNF3                  (14)
#define GPIOG_CRL_MODE4                 (16)
#define GPIOG_CRL_CNF4                  (18)
#define GPIOG_CRL_MODE5                 (20)
#define GPIOG_CRL_CNF5                  (22)
#define GPIOG_CRL_MODE6                 (24)
#define GPIOG_CRL_CNF6                  (26)
#define GPIOG_CRL_MODE7                 (28)
#define GPIOG_CRL_CNF7                  (30)
#define GPIOG_CRH                       SFR_IO32(GPIOG_BASE_ADDR + 0x004)
#define GPIOG_CRH_MODE8                 (0)
#define GPIOG_CRH_CNF8                  (2)
#define GPIOG_CRH_MODE9                 (4)
#define GPIOG_CRH_CNF9                  (6)
#define GPIOG_CRH_MODE10                (8)
#define GPIOG_CRH_CNF10                 (10)
#define GPIOG_CRH_MODE11                (12)
#define GPIOG_CRH_CNF11                 (14)
#define GPIOG_CRH_MODE12                (16)
#define GPIOG_CRH_CNF12                 (18)
#define GPIOG_CRH_MODE13                (20)
#define GPIOG_CRH_CNF13                 (22)
#define GPIOG_CRH_MODE14                (24)
#define GPIOG_CRH_CNF14                 (26)
#define GPIOG_CRH_MODE15                (28)
#define GPIOG_CRH_CNF15                 (30)
#define GPIOG_IDR                       SFR_IO32(GPIOG_BASE_ADDR + 0x008)
#define GPIOG_ODR                       SFR_IO32(GPIOG_BASE_ADDR + 0x00C)
#define GPIOG_BSRR                      SFR_IO32(GPIOG_BASE_ADDR + 0x010)
#define GPIOG_BRR                       SFR_IO32(GPIOG_BASE_ADDR + 0x014)
#define GPIOG_LCKR                      SFR_IO32(GPIOG_BASE_ADDR + 0x018)
// GPIO Port F
#define GPIOF_BASE_ADDR                 (0x40011C00)
#define GPIOF_CRL                       SFR_IO32(GPIOF_BASE_ADDR + 0x000)
#define GPIOF_CRL_MODE0                 (0)
#define GPIOF_CRL_CNF0                  (2)
#define GPIOF_CRL_MODE1                 (4)
#define GPIOF_CRL_CNF1                  (6)
#define GPIOF_CRL_MODE2                 (8)
#define GPIOF_CRL_CNF2                  (10)
#define GPIOF_CRL_MODE3                 (12)
#define GPIOF_CRL_CNF3                  (14)
#define GPIOF_CRL_MODE4                 (16)
#define GPIOF_CRL_CNF4                  (18)
#define GPIOF_CRL_MODE5                 (20)
#define GPIOF_CRL_CNF5                  (22)
#define GPIOF_CRL_MODE6                 (24)
#define GPIOF_CRL_CNF6                  (26)
#define GPIOF_CRL_MODE7                 (28)
#define GPIOF_CRL_CNF7                  (30)
#define GPIOF_CRH                       SFR_IO32(GPIOF_BASE_ADDR + 0x004)
#define GPIOF_CRH_MODE8                 (0)
#define GPIOF_CRH_CNF8                  (2)
#define GPIOF_CRH_MODE9                 (4)
#define GPIOF_CRH_CNF9                  (6)
#define GPIOF_CRH_MODE10                (8)
#define GPIOF_CRH_CNF10                 (10)
#define GPIOF_CRH_MODE11                (12)
#define GPIOF_CRH_CNF11                 (14)
#define GPIOF_CRH_MODE12                (16)
#define GPIOF_CRH_CNF12                 (18)
#define GPIOF_CRH_MODE13                (20)
#define GPIOF_CRH_CNF13                 (22)
#define GPIOF_CRH_MODE14                (24)
#define GPIOF_CRH_CNF14                 (26)
#define GPIOF_CRH_MODE15                (28)
#define GPIOF_CRH_CNF15                 (30)
#define GPIOF_IDR                       SFR_IO32(GPIOF_BASE_ADDR + 0x008)
#define GPIOF_ODR                       SFR_IO32(GPIOF_BASE_ADDR + 0x00C)
#define GPIOF_BSRR                      SFR_IO32(GPIOF_BASE_ADDR + 0x010)
#define GPIOF_BRR                       SFR_IO32(GPIOF_BASE_ADDR + 0x014)
#define GPIOF_LCKR                      SFR_IO32(GPIOF_BASE_ADDR + 0x018)
// GPIO Port E
#define GPIOE_BASE_ADDR                 (0x40011800)
#define GPIOE_CRL                       SFR_IO32(GPIOE_BASE_ADDR + 0x000)
#define GPIOE_CRL_MODE0                 (0)
#define GPIOE_CRL_CNF0                  (2)
#define GPIOE_CRL_MODE1                 (4)
#define GPIOE_CRL_CNF1                  (6)
#define GPIOE_CRL_MODE2                 (8)
#define GPIOE_CRL_CNF2                  (10)
#define GPIOE_CRL_MODE3                 (12)
#define GPIOE_CRL_CNF3                  (14)
#define GPIOE_CRL_MODE4                 (16)
#define GPIOE_CRL_CNF4                  (18)
#define GPIOE_CRL_MODE5                 (20)
#define GPIOE_CRL_CNF5                  (22)
#define GPIOE_CRL_MODE6                 (24)
#define GPIOE_CRL_CNF6                  (26)
#define GPIOE_CRL_MODE7                 (28)
#define GPIOE_CRL_CNF7                  (30)
#define GPIOE_CRH                       SFR_IO32(GPIOE_BASE_ADDR + 0x004)
#define GPIOE_CRH_MODE8                 (0)
#define GPIOE_CRH_CNF8                  (2)
#define GPIOE_CRH_MODE9                 (4)
#define GPIOE_CRH_CNF9                  (6)
#define GPIOE_CRH_MODE10                (8)
#define GPIOE_CRH_CNF10                 (10)
#define GPIOE_CRH_MODE11                (12)
#define GPIOE_CRH_CNF11                 (14)
#define GPIOE_CRH_MODE12                (16)
#define GPIOE_CRH_CNF12                 (18)
#define GPIOE_CRH_MODE13                (20)
#define GPIOE_CRH_CNF13                 (22)
#define GPIOE_CRH_MODE14                (24)
#define GPIOE_CRH_CNF14                 (26)
#define GPIOE_CRH_MODE15                (28)
#define GPIOE_CRH_CNF15                 (30)
#define GPIOE_IDR                       SFR_IO32(GPIOE_BASE_ADDR + 0x008)
#define GPIOE_ODR                       SFR_IO32(GPIOE_BASE_ADDR + 0x00C)
#define GPIOE_BSRR                      SFR_IO32(GPIOE_BASE_ADDR + 0x010)
#define GPIOE_BRR                       SFR_IO32(GPIOE_BASE_ADDR + 0x014)
#define GPIOE_LCKR                      SFR_IO32(GPIOE_BASE_ADDR + 0x018)
// GPIO Port D
#define GPIOD_BASE_ADDR                 (0x40011400)
#define GPIOD_CRL                       SFR_IO32(GPIOD_BASE_ADDR + 0x000)
#define GPIOD_CRL_MODE0                 (0)
#define GPIOD_CRL_CNF0                  (2)
#define GPIOD_CRL_MODE1                 (4)
#define GPIOD_CRL_CNF1                  (6)
#define GPIOD_CRL_MODE2                 (8)
#define GPIOD_CRL_CNF2                  (10)
#define GPIOD_CRL_MODE3                 (12)
#define GPIOD_CRL_CNF3                  (14)
#define GPIOD_CRL_MODE4                 (16)
#define GPIOD_CRL_CNF4                  (18)
#define GPIOD_CRL_MODE5                 (20)
#define GPIOD_CRL_CNF5                  (22)
#define GPIOD_CRL_MODE6                 (24)
#define GPIOD_CRL_CNF6                  (26)
#define GPIOD_CRL_MODE7                 (28)
#define GPIOD_CRL_CNF7                  (30)
#define GPIOD_CRH                       SFR_IO32(GPIOD_BASE_ADDR + 0x004)
#define GPIOD_CRH_MODE8                 (0)
#define GPIOD_CRH_CNF8                  (2)
#define GPIOD_CRH_MODE9                 (4)
#define GPIOD_CRH_CNF9                  (6)
#define GPIOD_CRH_MODE10                (8)
#define GPIOD_CRH_CNF10                 (10)
#define GPIOD_CRH_MODE11                (12)
#define GPIOD_CRH_CNF11                 (14)
#define GPIOD_CRH_MODE12                (16)
#define GPIOD_CRH_CNF12                 (18)
#define GPIOD_CRH_MODE13                (20)
#define GPIOD_CRH_CNF13                 (22)
#define GPIOD_CRH_MODE14                (24)
#define GPIOD_CRH_CNF14                 (26)
#define GPIOD_CRH_MODE15                (28)
#define GPIOD_CRH_CNF15                 (30)
#define GPIOD_IDR                       SFR_IO32(GPIOD_BASE_ADDR + 0x008)
#define GPIOD_ODR                       SFR_IO32(GPIOD_BASE_ADDR + 0x00C)
#define GPIOD_BSRR                      SFR_IO32(GPIOD_BASE_ADDR + 0x010)
#define GPIOD_BRR                       SFR_IO32(GPIOD_BASE_ADDR + 0x014)
#define GPIOD_LCKR                      SFR_IO32(GPIOD_BASE_ADDR + 0x018)
// GPIO Port C
#define GPIOC_BASE_ADDR                 (0x40011000)
#define GPIOC_CRL                       SFR_IO32(GPIOC_BASE_ADDR + 0x000)
#define GPIOC_CRL_MODE0                 (0)
#define GPIOC_CRL_CNF0                  (2)
#define GPIOC_CRL_MODE1                 (4)
#define GPIOC_CRL_CNF1                  (6)
#define GPIOC_CRL_MODE2                 (8)
#define GPIOC_CRL_CNF2                  (10)
#define GPIOC_CRL_MODE3                 (12)
#define GPIOC_CRL_CNF3                  (14)
#define GPIOC_CRL_MODE4                 (16)
#define GPIOC_CRL_CNF4                  (18)
#define GPIOC_CRL_MODE5                 (20)
#define GPIOC_CRL_CNF5                  (22)
#define GPIOC_CRL_MODE6                 (24)
#define GPIOC_CRL_CNF6                  (26)
#define GPIOC_CRL_MODE7                 (28)
#define GPIOC_CRL_CNF7                  (30)
#define GPIOC_CRH                       SFR_IO32(GPIOC_BASE_ADDR + 0x004)
#define GPIOC_CRH_MODE8                 (0)
#define GPIOC_CRH_CNF8                  (2)
#define GPIOC_CRH_MODE9                 (4)
#define GPIOC_CRH_CNF9                  (6)
#define GPIOC_CRH_MODE10                (8)
#define GPIOC_CRH_CNF10                 (10)
#define GPIOC_CRH_MODE11                (12)
#define GPIOC_CRH_CNF11                 (14)
#define GPIOC_CRH_MODE12                (16)
#define GPIOC_CRH_CNF12                 (18)
#define GPIOC_CRH_MODE13                (20)
#define GPIOC_CRH_CNF13                 (22)
#define GPIOC_CRH_MODE14                (24)
#define GPIOC_CRH_CNF14                 (26)
#define GPIOC_CRH_MODE15                (28)
#define GPIOC_CRH_CNF15                 (30)
#define GPIOC_IDR                       SFR_IO32(GPIOC_BASE_ADDR + 0x008)
#define GPIOC_ODR                       SFR_IO32(GPIOC_BASE_ADDR + 0x00C)
#define GPIOC_BSRR                      SFR_IO32(GPIOC_BASE_ADDR + 0x010)
#define GPIOC_BRR                       SFR_IO32(GPIOC_BASE_ADDR + 0x014)
#define GPIOC_LCKR                      SFR_IO32(GPIOC_BASE_ADDR + 0x018)
// GPIO Port B
#define GPIOB_BASE_ADDR                 (0x40010C00)
#define GPIOB_CRL                       SFR_IO32(GPIOB_BASE_ADDR + 0x000)
#define GPIOB_CRL_MODE0                 (0)
#define GPIOB_CRL_CNF0                  (2)
#define GPIOB_CRL_MODE1                 (4)
#define GPIOB_CRL_CNF1                  (6)
#define GPIOB_CRL_MODE2                 (8)
#define GPIOB_CRL_CNF2                  (10)
#define GPIOB_CRL_MODE3                 (12)
#define GPIOB_CRL_CNF3                  (14)
#define GPIOB_CRL_MODE4                 (16)
#define GPIOB_CRL_CNF4                  (18)
#define GPIOB_CRL_MODE5                 (20)
#define GPIOB_CRL_CNF5                  (22)
#define GPIOB_CRL_MODE6                 (24)
#define GPIOB_CRL_CNF6                  (26)
#define GPIOB_CRL_MODE7                 (28)
#define GPIOB_CRL_CNF7                  (30)
#define GPIOB_CRH                       SFR_IO32(GPIOB_BASE_ADDR + 0x004)
#define GPIOB_CRH_MODE8                 (0)
#define GPIOB_CRH_CNF8                  (2)
#define GPIOB_CRH_MODE9                 (4)
#define GPIOB_CRH_CNF9                  (6)
#define GPIOB_CRH_MODE10                (8)
#define GPIOB_CRH_CNF10                 (10)
#define GPIOB_CRH_MODE11                (12)
#define GPIOB_CRH_CNF11                 (14)
#define GPIOB_CRH_MODE12                (16)
#define GPIOB_CRH_CNF12                 (18)
#define GPIOB_CRH_MODE13                (20)
#define GPIOB_CRH_CNF13                 (22)
#define GPIOB_CRH_MODE14                (24)
#define GPIOB_CRH_CNF14                 (26)
#define GPIOB_CRH_MODE15                (28)
#define GPIOB_CRH_CNF15                 (30)
#define GPIOB_IDR                       SFR_IO32(GPIOB_BASE_ADDR + 0x008)
#define GPIOB_ODR                       SFR_IO32(GPIOB_BASE_ADDR + 0x00C)
#define GPIOB_BSRR                      SFR_IO32(GPIOB_BASE_ADDR + 0x010)
#define GPIOB_BRR                       SFR_IO32(GPIOB_BASE_ADDR + 0x014)
#define GPIOB_LCKR                      SFR_IO32(GPIOB_BASE_ADDR + 0x018)
// GPIO Port A
#define GPIOA_BASE_ADDR                 (0x40010800)
#define GPIOA_CRL                       SFR_IO32(GPIOA_BASE_ADDR + 0x000)
#define GPIOA_CRL_MODE0                 (0)
#define GPIOA_CRL_CNF0                  (2)
#define GPIOA_CRL_MODE1                 (4)
#define GPIOA_CRL_CNF1                  (6)
#define GPIOA_CRL_MODE2                 (8)
#define GPIOA_CRL_CNF2                  (10)
#define GPIOA_CRL_MODE3                 (12)
#define GPIOA_CRL_CNF3                  (14)
#define GPIOA_CRL_MODE4                 (16)
#define GPIOA_CRL_CNF4                  (18)
#define GPIOA_CRL_MODE5                 (20)
#define GPIOA_CRL_CNF5                  (22)
#define GPIOA_CRL_MODE6                 (24)
#define GPIOA_CRL_CNF6                  (26)
#define GPIOA_CRL_MODE7                 (28)
#define GPIOA_CRL_CNF7                  (30)
#define GPIOA_CRH                       SFR_IO32(GPIOA_BASE_ADDR + 0x004)
#define GPIOA_CRH_MODE8                 (0)
#define GPIOA_CRH_CNF8                  (2)
#define GPIOA_CRH_MODE9                 (4)
#define GPIOA_CRH_CNF9                  (6)
#define GPIOA_CRH_MODE10                (8)
#define GPIOA_CRH_CNF10                 (10)
#define GPIOA_CRH_MODE11                (12)
#define GPIOA_CRH_CNF11                 (14)
#define GPIOA_CRH_MODE12                (16)
#define GPIOA_CRH_CNF12                 (18)
#define GPIOA_CRH_MODE13                (20)
#define GPIOA_CRH_CNF13                 (22)
#define GPIOA_CRH_MODE14                (24)
#define GPIOA_CRH_CNF14                 (26)
#define GPIOA_CRH_MODE15                (28)
#define GPIOA_CRH_CNF15                 (30)
#define GPIOA_IDR                       SFR_IO32(GPIOA_BASE_ADDR + 0x008)
#define GPIOA_ODR                       SFR_IO32(GPIOA_BASE_ADDR + 0x00C)
#define GPIOA_BSRR                      SFR_IO32(GPIOA_BASE_ADDR + 0x010)
#define GPIOA_BRR                       SFR_IO32(GPIOA_BASE_ADDR + 0x014)
#define GPIOA_LCKR                      SFR_IO32(GPIOA_BASE_ADDR + 0x018)
// EXTI
#define EXTI_BASE_ADDR                  (0x40010400)
// AFIO
#define AFIO_BASE_ADDR                  (0x40010000)
#define AFIO_EVCR                       SFR_IO32(AFIO_BASE_ADDR + 0x000)
#define AFIO_EVCR_PIN                   (0)
#define AFIO_EVCR_PORT                  (3)
#define AFIO_EVCR_EVoE                  (6)
#define AFIO_MAPR                       SFR_IO32(AFIO_BASE_ADDR + 0x004)
#define AFIO_MAPR_SPI1_REMAP            (0)
#define AFIO_MAPR_I2C1_REMAP            (1)
#define AFIO_MAPR_USART1_REMAP          (2)
#define AFIO_MAPR_USART2_REMAP          (3)
#define AFIO_MAPR_USART3_REMAP          (4)
#define AFIO_MAPR_TIM1_REMAP            (6)
#define AFIO_MAPR_TIM2_REMAP            (8)
#define AFIO_MAPR_TIM3_REMAP            (10)
#define AFIO_MAPR_TIM4_REMAP            (12)
#define AFIO_MAPR_CAN1_REMAP            (13)
#define AFIO_MAPR_PD01_REMAP            (15)
#define AFIO_MAPR_TIM5CH4_REMAP         (16)
#define AFIO_MAPR_ADC1_ETRGINJ_REMAP    (17)
#define AFIO_MAPR_ADC1_ETRGREG_REMAP    (18)
#define AFIO_MAPR_ADC2_ETRGINJ_REMAP    (19)
#define AFIO_MAPR_ADC2_ETRGREG_REMAP    (20)
#define AFIO_MAPR_ETH_REMAP             (21)
#define AFIO_MAPR_CAN2_REMAP            (22)
#define AFIO_MAPR_MII_RMII_SEL          (23)
#define AFIO_MAPR_SWJ_CFG               (24)
#define AFIO_MAPR_SPI3_REMAP            (28)
#define AFIO_MAPR_TIM2ITR1_IREMAP       (29)
#define AFIO_MAPR_PTP_PPS_REMAP         (30)
#define AFIO_EXTICR1                    SFR_IO32(AFIO_BASE_ADDR + 0x008)
#define AFIO_EXTICR2                    SFR_IO32(AFIO_BASE_ADDR + 0x00C)
#define AFIO_EXTICR3                    SFR_IO32(AFIO_BASE_ADDR + 0x010)
#define AFIO_EXTICR4                    SFR_IO32(AFIO_BASE_ADDR + 0x014)
// DAC
#define DAC_BASE_ADDR                   (0x40007400)
#define DAC_CR                          SFR_IO32(DAC_BASE_ADDR + 0x000)
#define DAC_CR_EN1                      (0)
#define DAC_CR_BOFF1                    (1)
#define DAC_CR_TEN1                     (2)
#define DAC_CR_TSEL1                    (3)
#define DAC_CR_WAVE1                    (6)
#define DAC_CR_MAMP1                    (8)
#define DAC_CR_DMAEN1                   (12)
#define DAC_CR_EN2                      (16)
#define DAC_CR_BOFF2                    (17)
#define DAC_CR_TEN2                     (18)
#define DAC_CR_TSEL2                    (19)
#define DAC_CR_WAVE2                    (22)
#define DAC_CR_MAMP2                    (24)
#define DAC_CR_DMAEN2                   (28)
#define DAC_SWTRIGR                     SFR_IO32(DAC_BASE_ADDR + 0x004)
#define DAC_SWTRIGR_SWTRIG1             (0)
#define DAC_SWTRIGR_SWTRIG2             (1)
#define DAC_DHR12R1                     SFR_IO32(DAC_BASE_ADDR + 0x008)
#define DAC_DHR12L1                     SFR_IO32(DAC_BASE_ADDR + 0x00C)
#define DAC_DHR8R1                      SFR_IO32(DAC_BASE_ADDR + 0x010)
#define DAC_DHR12R2                     SFR_IO32(DAC_BASE_ADDR + 0x014)
#define DAC_DHR12L2                     SFR_IO32(DAC_BASE_ADDR + 0x018)
#define DAC_DHR8R2                      SFR_IO32(DAC_BASE_ADDR + 0x01C)
#define DAC_DHR12RD                     SFR_IO32(DAC_BASE_ADDR + 0x020)
#define DAC_DHR12LD                     SFR_IO32(DAC_BASE_ADDR + 0x024)
#define DAC_DHR8RD                      SFR_IO32(DAC_BASE_ADDR + 0x028)
#define DAC_DOR1                        SFR_IO32(DAC_BASE_ADDR + 0x02C)
#define DAC_DOR2                        SFR_IO32(DAC_BASE_ADDR + 0x030)
// Power control
#define PWR_BASE_ADDR                   (0x40007000)
// Backup registers
#define BKP_BASE_ADDR                   (0x40006C00)
// bxCAN2
#define CAN2_BASE_ADDR                  (0x40006800)
// bxCAN1
#define CAN1_BASE_ADDR                  (0x40006400)
// Shared USB/CAN SRAM 512 bytes
#define USB_CAN_SRAM_BASE_ADDR          (0x40006000)
// USB device FS registers
#define USB_FS_BASE_ADDR                (0x40005C00)
// I2C2
#define I2C2_BASE_ADDR                  (0x40005800)
// I2C1
#define I2C1_BASE_ADDR                  (0x40005400)
// UART5
#define UART5_BASE_ADDR                 (0x40005000)
// UART4
#define UART4_BASE_ADDR                 (0x40004C00)
// USART3
#define USART3_BASE_ADDR                (0x40004800)
// USART2
#define USART2_BASE_ADDR                (0x40004400)
#define USART2_SR                       SFR_IO32(USART2_BASE_ADDR + 0x000)
#define USART2_SR_PE                    (0)
#define USART2_SR_FE                    (1)
#define USART2_SR_NE                    (2)
#define USART2_SR_ORE                   (3)
#define USART2_SR_IDLE                  (4)
#define USART2_SR_RXNE                  (5)
#define USART2_SR_TC                    (6)
#define USART2_SR_TXE                   (7)
#define USART2_SR_LBD                   (8)
#define USART2_SR_CTS                   (9)
#define USART2_DR                       SFR_IO32(USART2_BASE_ADDR + 0x004)
#define USART2_BRR                      SFR_IO32(USART2_BASE_ADDR + 0x008)
#define USART2_BRR_DIV_Fraction         (0)
#define USART2_BRR_DIV_Mantissa         (4)
#define USART2_CR1                      SFR_IO32(USART2_BASE_ADDR + 0x00C)
#define USART2_CR1_SBK                  (0)
#define USART2_CR1_RWU                  (1)
#define USART2_CR1_RE                   (2)
#define USART2_CR1_TE                   (3)
#define USART2_CR1_IDLEIE               (4)
#define USART2_CR1_RXNEIE               (5)
#define USART2_CR1_TCIE                 (6)
#define USART2_CR1_TXEIE                (7)
#define USART2_CR1_PEIE                 (8)
#define USART2_CR1_PS                   (9)
#define USART2_CR1_PCE                  (10)
#define USART2_CR1_WAKE                 (11)
#define USART2_CR1_M                    (12)
#define USART2_CR1_UE                   (13)
#define USART2_CR2                      SFR_IO32(USART2_BASE_ADDR + 0x010)
#define USART2_CR2_ADD                  (0)
#define USART2_CR2_LBDL                 (5)
#define USART2_CR2_LBDIE                (6)
#define USART2_CR2_LBCL                 (8)
#define USART2_CR2_CPHA                 (9)
#define USART2_CR2_CPOL                 (10)
#define USART2_CR2_CLKEN                (11)
#define USART2_CR2_STOP                 (12)
#define USART2_CR2_LINEN                (14)
#define USART2_CR3                      SFR_IO32(USART2_BASE_ADDR + 0x014)
#define USART2_CR3_EIE                  (0)
#define USART2_CR3_IREN                 (1)
#define USART2_CR3_IRLP                 (2)
#define USART2_CR3_HDSEL                (3)
#define USART2_CR3_NACK                 (4)
#define USART2_CR3_SCEN                 (5)
#define USART2_CR3_DMAR                 (6)
#define USART2_CR3_DMAT                 (7)
#define USART2_CR3_RTSE                 (8)
#define USART2_CR3_CTSE                 (9)
#define USART2_CR3_CTSIE                (10)
#define USART2_GTPR                     SFR_IO32(USART2_BASE_ADDR + 0x018)
#define USART2_GTPR_PSC                 (0)
#define USART2_GTPR_GT                  (8)
// SPI3/I2S
#define SPI3_BASE_ADDR                  (0x40003C00)
#define SPI3_CR1                        SFR_IO32(SPI3_BASE_ADDR + 0x000)
#define SPI3_CR1_CPHA                   (0)
#define SPI3_CR1_CPOL                   (1)
#define SPI3_CR1_MSTR                   (2)
#define SPI3_CR1_BR                     (3)
#define SPI3_CR1_BR_bp                  (3)
#define SPI3_CR1_BR_PCLK2_gc            (0 << SPI3_CR1_BR_bp)
#define SPI3_CR1_BR_PCLK4_gc            (1 << SPI3_CR1_BR_bp)
#define SPI3_CR1_BR_PCLK8_gc            (2 << SPI3_CR1_BR_bp)
#define SPI3_CR1_BR_PCLK16_gc           (3 << SPI3_CR1_BR_bp)
#define SPI3_CR1_BR_PCLK32_gc           (4 << SPI3_CR1_BR_bp)
#define SPI3_CR1_BR_PCLK64_gc           (5 << SPI3_CR1_BR_bp)
#define SPI3_CR1_BR_PCLK128_gc          (6 << SPI3_CR1_BR_bp)
#define SPI3_CR1_BR_PCLK256_gc          (7 << SPI3_CR1_BR_bp)
#define SPI3_CR1_SPE                    (6)
#define SPI3_CR1_LSBFIRST               (7)
#define SPI3_CR1_SSI                    (8)
#define SPI3_CR1_SSM                    (9)
#define SPI3_CR1_RXONLY                 (10)
#define SPI3_CR1_DFF                    (11)
#define SPI3_CR1_CRCNEXT                (12)
#define SPI3_CR1_CRCEN                  (13)
#define SPI3_CR1_BIDIOE                 (14)
#define SPI3_CR1_BIDIMODE               (15)
#define SPI3_CR2                        SFR_IO32(SPI3_BASE_ADDR + 0x004)
#define SPI3_CR2_RXDMAEN                (0)
#define SPI3_CR2_TXDMAEN                (1)
#define SPI3_CR2_SSOE                   (2)
#define SPI3_CR2_ERRIE                  (5)
#define SPI3_CR2_RXNEIE                 (6)
#define SPI3_CR2_TXEIE                  (7)
#define SPI3_SR                         SFR_IO32(SPI3_BASE_ADDR + 0x008)
#define SPI3_SR_RXNE                    (0)
#define SPI3_SR_TXE                     (1)
#define SPI3_SR_CHSIDE                  (2)
#define SPI3_SR_UDR                     (3)
#define SPI3_SR_CRCERR                  (4)
#define SPI3_SR_MODF                    (5)
#define SPI3_SR_OVR                     (6)
#define SPI3_SR_BSY                     (7)
#define SPI3_DR                         SFR_IO32(SPI3_BASE_ADDR + 0x00C)
#define SPI3_CRCPR                      SFR_IO32(SPI3_BASE_ADDR + 0x010)
#define SPI3_RXCRCR                     SFR_IO32(SPI3_BASE_ADDR + 0x014)
#define SPI3_TXCRCR                     SFR_IO32(SPI3_BASE_ADDR + 0x018)
#define SPI3_I2SCFGR                    SFR_IO32(SPI3_BASE_ADDR + 0x01C)
#define SPI3_I2SPR                      SFR_IO32(SPI3_BASE_ADDR + 0x020)
// SPI2/I2S
#define SPI2_BASE_ADDR                  (0x40003800)
// Independent watchdog
#define IWDG_BASE_ADDR                  (0x40003000)
// Window watchdog
#define WWDG_BASE_ADDR                  (0x40002C00)
// RTC
#define RTC_BASE_ADDR                   (0x40002800)
// TIM7 timer
#define TIM7_BASE_ADDR                  (0x40001400)
// TIM6 timer
#define TIM6_BASE_ADDR                  (0x40001000)
// TIM5 timer
#define TIM5_BASE_ADDR                  (0x40000C00)
// TIM4 timer
#define TIM4_BASE_ADDR                  (0x40000800)
// TIM3 timer
#define TIM3_BASE_ADDR                  (0x40000400)
// TIM2 timer
#define TIM2_BASE_ADDR                  (0x40000000)

// Oscillators
#define LSI_OSC                         (40000UL)
#define LSE_OSC                         (32768UL)
#define HSI_OSC                         (8000000UL)

#endif // STM32F107_H

